/* ===================================
Bodenfeuchtexxxxxx.ino
Programm für einen ESP32 mit kapazitivem Bodenfeuchtesensor
Autoren: Janek Westphal, Claus Brell
Stand: 15.01.2023

Zum Einsatz im Rahmen der Schulkooperationen des FB Wirtschaftswissenschaften
Hochschule Niederhein
========================================*/

// Bibliothek für den Plattform-Zugriff
#include <WiFi.h>

// Wroom-Modul Anschluss G35 = GPIO5
// ist der verfügbare ADC1
#define SENSOR_PIN 35
// Netzwerk Parameter
const char* ssid     = "iot";
const char* password = "1234abcd5678";
//const char* ssid     = "Gloryhammer";
//const char* password = "dada08154711";

// Parameter für die Biene40 Plattform
// für jeden Sensor unterschiedlich
String SENSOR_NR   = "66600021";
String UPLOAD_CODE = "62354821";

// Nutzerbeglückung
// Definition der Konstante LED mit dem Wert 12
#define LED 32


void setup()
{
    // Set pin mode
    pinMode(LED,OUTPUT);
    blinky(1,1000);

    Serial.begin(9600);
    delay(10);
    
    WiFi.begin(ssid, password);
    while (WiFi.status() != WL_CONNECTED) {
        delay(500);
        Serial.print(".");
    }
    Serial.println();
    Serial.println(WiFi.localIP());
}


void loop()
{
    WiFiClient client;
    if (!client.connect("210301.de", 80)) {
        Serial.println("Verbindung fehlgeschlagen");
        delay(15000);
        blinky(7,64);
        return;
    }
    // 66600022 Testsystem Bodenfeuchte Brell
    int measure = analogRead(SENSOR_PIN);
    int percent = convertMeasureToPercent(measure);
    // Parameer für http-Aufruf zusammenmstellen
    // API für die Biene40-Plattform
    String url = "/b40/WS/msg220304.php?x="+SENSOR_NR+";"+UPLOAD_CODE+";-10;";
    url += percent;

    client.print(String("GET ") + url + " HTTP/1.1\r\n" +
                 "Host: " + "210301.de" + "\r\n" +
                 "Connection: close\r\n\r\n");
    unsigned long timeout = millis();
    while (client.available() == 0) {
        if (millis() - timeout > 5000) {
            Serial.println("Timeout");
            client.stop();
            WiFi.disconnect();
            WiFi.reconnect();
            return;
        }
    // Nutzerbeglückung: die ersten 100 Zeichen der http-Antwort ausgeben
    int i=0;
    if (client.available()) blinky(1,1000);
    while(client.available() && i<100) {
       // read an incoming byte from the server and print them to serial monitor:
       char c = client.read();
       Serial.print(c);
       i++;
    }
    }

    Serial.println(url);
    // Pause zwischen den Messungen
    Serial.println("===================\r\nPause ");
    for (int i=0;i<6;i++){
      blinky(1,64);
      Serial.print('.');    
      delay(10000);
    } // Ende for
}

/* ------------------------------
   Funktionen
   ------------------------------*/

// Messwert in den Bereich 0-100 Transformieren
int convertMeasureToPercent(int measure){
  /* Es wird von einem 12 Bit Wert ausgegangen.
     0 entstricht 100%
     4095 entspricht 0%
     Dazwischen wird linear umgerechnet.
     Das ist nicht präzise, genügt aber für den
     Anwendungsfall Bodenfeuchtemessung
  */
  float quotient=1-(float)measure/4095.0;
  // WARTUNG
  // Serial.print("transformierter Messwert: ");
  Serial.print("transformierter Wert: ");
  Serial.println(quotient);  
  int percent=(int)(quotient*100);
  return percent;   
} // Ende Funktion

// Rückmeldung für den Nutzer
void blinky(int anzahl, int dauer){
  /* Die interne LED wird zur Signalisierung genutzt
     1 Sekunde - alles ok
     64 ms Blitz - ich arbeite
     5 * halbe Sekunde - etwas Schlimmes ist passiert
  */
  for (int i=0;i<anzahl;i++){
    delay(500);
    digitalWrite(LED,HIGH);
    delay(dauer);
    digitalWrite(LED,LOW);
  } // Ende for   
} // Ende Funktion blinky
